@@ -20,6 +20,7 @@ from .items import Key
 from .items import Null
 from .items import SingleKey
 from .items import Table
+from .items import Trivia
 from .items import Whitespace
 from .items import _CustomDict
 from .items import item as _item
@@ -31,6 +32,8 @@ _NOT_SET = object()
 class Container(_CustomDict):
     """
     A container for items within a TOMLDocument.
+
+    This class implements the `dict` interface with copy/deepcopy protocol.
     """
 
     def __init__(self, parsed: bool = False) -> None:
@@ -78,6 +81,13 @@ class Container(_CustomDict):
     ) -> "Container":
         """
         Adds an item to the current Container.
+
+        :Example:
+
+        >>> # add a key-value pair
+        >>> doc.add('key', 'value')
+        >>> # add a comment or whitespace or newline
+        >>> doc.add(comment('# comment'))
         """
         if item is None:
             if not isinstance(key, (Comment, Whitespace)):
@@ -89,13 +99,76 @@ class Container(_CustomDict):
 
         return self.append(key, item)
 
+    def _handle_dotted_key(self, key: Key, value: Item) -> None:
+        names = tuple(iter(key))
+        name = names[0]
+        name._dotted = True
+        if name in self:
+            if not isinstance(value, Table):
+                table = Table(Container(True), Trivia(), False, is_super_table=True)
+                _table = table
+                for i, _name in enumerate(names[1:]):
+                    if i == len(names) - 2:
+                        _name.sep = key.sep
+
+                        _table.append(_name, value)
+                    else:
+                        _name._dotted = True
+                        _table.append(
+                            _name,
+                            Table(
+                                Container(True),
+                                Trivia(),
+                                False,
+                                is_super_table=i < len(names) - 2,
+                            ),
+                        )
+
+                        _table = _table[_name]
+
+                value = table
+
+            self.append(name, value)
+
+            return
+        else:
+            table = Table(Container(True), Trivia(), False, is_super_table=True)
+            self.append(name, table)
+
+        for i, _name in enumerate(names[1:]):
+            if i == len(names) - 2:
+                _name.sep = key.sep
+
+                table.append(_name, value)
+            else:
+                _name._dotted = True
+                if _name in table.value:
+                    table = table.value[_name]
+                else:
+                    table.append(
+                        _name,
+                        Table(
+                            Container(True),
+                            Trivia(),
+                            False,
+                            is_super_table=i < len(names) - 2,
+                        ),
+                    )
+
+                    table = table[_name]
+
     def append(self, key: Union[Key, str, None], item: Item) -> "Container":
+        """Similar to :meth:`add` but both key and value must be given."""
         if not isinstance(key, Key) and key is not None:
             key = SingleKey(key)
 
         if not isinstance(item, Item):
             item = _item(item)
 
+        if key is not None and key.is_multi():
+            self._handle_dotted_key(key, item)
+            return self
+
         if isinstance(item, (AoT, Table)) and item.name is None:
             item.name = key.key
 
@@ -255,6 +328,7 @@ class Container(_CustomDict):
         return self
 
     def remove(self, key: Union[Key, str]) -> "Container":
+        """Remove a key from the container."""
         if not isinstance(key, Key):
             key = SingleKey(key)
 
@@ -361,6 +435,7 @@ class Container(_CustomDict):
         return self
 
     def item(self, key: Union[Key, str]) -> Item:
+        """Get an item for the given key."""
         if not isinstance(key, Key):
             key = SingleKey(key)
 
@@ -377,10 +452,12 @@ class Container(_CustomDict):
         return self._body[idx][1]
 
     def last_item(self) -> Optional[Item]:
+        """Get the last item."""
         if self._body:
             return self._body[-1][1]
 
     def as_string(self) -> str:
+        """Render as TOML string."""
         s = ""
         for k, v in self._body:
             if k is not None:
@@ -428,6 +505,8 @@ class Container(_CustomDict):
                 table.trivia.trail,
                 "\n" if "\n" not in table.trivia.trail and len(table.value) > 0 else "",
             )
+        elif table.trivia.indent == "\n":
+            cur += table.trivia.indent
 
         for k, v in table.value.body:
             if isinstance(v, Table):
@@ -542,7 +621,8 @@ class Container(_CustomDict):
 
     def __setitem__(self, key: Union[Key, str], value: Any) -> None:
         if key is not None and key in self:
-            self._replace(key, key, value)
+            old_key = next(filter(lambda k: k == key, self._map))
+            self._replace(old_key, key, value)
         else:
             self.append(key, value)
 
@@ -559,9 +639,6 @@ class Container(_CustomDict):
         if not isinstance(key, Key):
             key = SingleKey(key)
 
-        if not isinstance(new_key, Key):
-            new_key = SingleKey(new_key)
-
         idx = self._map.get(key, None)
         if idx is None:
             raise NonExistentKey(key)
@@ -571,8 +648,7 @@ class Container(_CustomDict):
     def _replace_at(
         self, idx: Union[int, Tuple[int]], new_key: Union[Key, str], value: Item
     ) -> None:
-        if not isinstance(new_key, Key):
-            new_key = SingleKey(new_key)
+        value = _item(value)
 
         if isinstance(idx, tuple):
             for i in idx[1:]:
@@ -581,17 +657,21 @@ class Container(_CustomDict):
             idx = idx[0]
 
         k, v = self._body[idx]
+        if not isinstance(new_key, Key):
+            if (
+                isinstance(value, (AoT, Table)) != isinstance(v, (AoT, Table))
+                or new_key != k.key
+            ):
+                new_key = SingleKey(new_key)
+            else:  # Inherit the sep of the old key
+                new_key = k
 
-        self._map[new_key] = self._map.pop(k)
+        del self._map[k]
+        self._map[new_key] = idx
         if new_key != k:
             dict.__delitem__(self, k)
 
-        if isinstance(self._map[new_key], tuple):
-            self._map[new_key] = self._map[new_key][0]
-
-        value = _item(value)
-
-        if isinstance(value, (AoT, Table)) and not isinstance(v, (AoT, Table)):
+        if isinstance(value, (AoT, Table)) != isinstance(v, (AoT, Table)):
             # new tables should appear after all non-table values
             self.remove(k)
             for i in range(idx, len(self._body)):
