@@ -1,17 +1,12 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import re
 import string
 
-from typing import Any
-from typing import Generator
 from typing import List
 from typing import Optional
 from typing import Tuple
+from typing import Type
 from typing import Union
 
-from ._compat import chr
 from ._compat import decode
 from ._utils import RFC_3339_LOOSE
 from ._utils import _escaped
@@ -44,6 +39,7 @@ from .items import Item
 from .items import Key
 from .items import KeyType
 from .items import Null
+from .items import SingleKey
 from .items import String
 from .items import StringType
 from .items import Table
@@ -67,11 +63,11 @@ class Parser:
     Parser for TOML documents.
     """
 
-    def __init__(self, string):  # type: (str) -> None
+    def __init__(self, string: str) -> None:
         # Input to parse
         self._src = Source(decode(string))
 
-        self._aot_stack = []
+        self._aot_stack: List[Key] = []
 
     @property
     def _state(self):
@@ -89,20 +85,20 @@ class Parser:
     def _marker(self):
         return self._src.marker
 
-    def extract(self):  # type: () -> str
+    def extract(self) -> str:
         """
         Extracts the value between marker and index
         """
         return self._src.extract()
 
-    def inc(self, exception=None):  # type: (Optional[ParseError.__class__]) -> bool
+    def inc(self, exception: Optional[Type[ParseError]] = None) -> bool:
         """
         Increments the parser if the end of the input has not been reached.
         Returns whether or not it was able to advance.
         """
         return self._src.inc(exception=exception)
 
-    def inc_n(self, n, exception=None):  # type: (int, Optional[ParseError]) -> bool
+    def inc_n(self, n: int, exception: Optional[Type[ParseError]] = None) -> bool:
         """
         Increments the parser by n characters
         if the end of the input has not been reached.
@@ -115,25 +111,25 @@ class Parser:
         """
         return self._src.consume(chars=chars, min=min, max=max)
 
-    def end(self):  # type: () -> bool
+    def end(self) -> bool:
         """
         Returns True if the parser has reached the end of the input.
         """
         return self._src.end()
 
-    def mark(self):  # type: () -> None
+    def mark(self) -> None:
         """
         Sets the marker to the index's current position
         """
         self._src.mark()
 
-    def parse_error(self, exception=ParseError, *args):
+    def parse_error(self, exception=ParseError, *args, **kwargs):
         """
         Creates a generic "parse error" at the current position.
         """
-        return self._src.parse_error(exception, *args)
+        return self._src.parse_error(exception, *args, **kwargs)
 
-    def parse(self):  # type: () -> TOMLDocument
+    def parse(self) -> TOMLDocument:
         body = TOMLDocument(True)
 
         # Take all keyvals outside of tables/AoT's.
@@ -148,10 +144,8 @@ class Parser:
                 break
 
             key, value = item
-            if key is not None and key.is_dotted():
+            if (key is not None and key.is_multi()) or not self._merge_ws(value, body):
                 # We actually have a table
-                self._handle_dotted_key(body, key, value)
-            elif not self._merge_ws(value, body):
                 body.append(key, value)
 
             self.mark()
@@ -161,7 +155,7 @@ class Parser:
             if isinstance(value, Table) and value.is_aot_element():
                 # This is just the first table in an AoT. Parse the rest of the array
                 # along with it.
-                value = self._parse_aot(value, key.key)
+                value = self._parse_aot(value, key)
 
             body.append(key, value)
 
@@ -169,7 +163,7 @@ class Parser:
 
         return body
 
-    def _merge_ws(self, item, container):  # type: (Item, Container) -> bool
+    def _merge_ws(self, item: Item, container: Container) -> bool:
         """
         Merges the given Item with the last one currently in the given Container if
         both are whitespace items.
@@ -191,85 +185,20 @@ class Parser:
 
         return True
 
-    def _is_child(self, parent, child):  # type: (str, str) -> bool
+    def _is_child(self, parent: Key, child: Key) -> bool:
         """
         Returns whether a key is strictly a child of another key.
         AoT siblings are not considered children of one another.
         """
-        parent_parts = tuple(self._split_table_name(parent))
-        child_parts = tuple(self._split_table_name(child))
+        parent_parts = tuple(parent)
+        child_parts = tuple(child)
 
         if parent_parts == child_parts:
             return False
 
         return parent_parts == child_parts[: len(parent_parts)]
 
-    def _split_table_name(self, name):  # type: (str) -> Generator[Key]
-        in_name = False
-        current = ""
-        t = KeyType.Bare
-        parts = 0
-        for c in name:
-            c = TOMLChar(c)
-
-            if c == ".":
-                if in_name:
-                    current += c
-                    continue
-
-                if not current:
-                    raise self.parse_error()
-
-                yield Key(current.strip(), t=t, sep="", original=current)
-
-                parts += 1
-
-                current = ""
-                t = KeyType.Bare
-                continue
-            elif c in {"'", '"'}:
-                if in_name:
-                    if (
-                        t == KeyType.Literal
-                        and c == '"'
-                        or t == KeyType.Basic
-                        and c == "'"
-                    ):
-                        current += c
-                        continue
-
-                    if c != t.value:
-                        raise self.parse_error()
-
-                    in_name = False
-                else:
-                    if (
-                        current.strip()
-                        and TOMLChar(current[-1]).is_spaces()
-                        and not parts
-                    ):
-                        raise self.parse_error()
-
-                    in_name = True
-                    t = KeyType.Literal if c == "'" else KeyType.Basic
-
-                continue
-            elif in_name or c.is_bare_key_char():
-                current += c
-            elif c.is_spaces():
-                # A space is only valid at this point
-                # if it's in between parts.
-                # We store it for now and will check
-                # later if it's valid
-                current += c
-                continue
-            else:
-                raise self.parse_error()
-
-        if current.strip():
-            yield Key(current.strip(), t=t, sep="", original=current)
-
-    def _parse_item(self):  # type: () -> Optional[Tuple[Optional[Key], Item]]
+    def _parse_item(self) -> Optional[Tuple[Optional[Key], Item]]:
         """
         Attempts to parse the next item and returns it, along with its key
         if the item is value-like.
@@ -297,7 +226,7 @@ class Parser:
                     # Found a table, delegate to the calling function.
                     return
                 else:
-                    # Begining of a KV pair.
+                    # Beginning of a KV pair.
                     # Return to beginning of whitespace so it gets included
                     # as indentation for the KV about to be parsed.
                     state.restore = True
@@ -305,7 +234,7 @@ class Parser:
 
         return self._parse_key_value(True)
 
-    def _parse_comment_trail(self):  # type: () -> Tuple[str, str, str]
+    def _parse_comment_trail(self, parse_trail: bool = True) -> Tuple[str, str, str]:
         """
         Returns (comment_ws, comment, trail)
         If there is no comment, comment_ws and comment will
@@ -350,22 +279,23 @@ class Parser:
             if self.end():
                 break
 
-        while self._current.is_spaces() and self.inc():
-            pass
-
-        if self._current == "\r":
-            self.inc()
-
-        if self._current == "\n":
-            self.inc()
-
         trail = ""
-        if self._idx != self._marker or self._current.is_ws():
-            trail = self.extract()
+        if parse_trail:
+            while self._current.is_spaces() and self.inc():
+                pass
+
+            if self._current == "\r":
+                self.inc()
+
+            if self._current == "\n":
+                self.inc()
+
+            if self._idx != self._marker or self._current.is_ws():
+                trail = self.extract()
 
         return comment_ws, comment, trail
 
-    def _parse_key_value(self, parse_comment=False):  # type: (bool) -> (Key, Item)
+    def _parse_key_value(self, parse_comment: bool = False) -> Tuple[Key, Item]:
         # Leading indent
         self.mark()
 
@@ -386,7 +316,8 @@ class Parser:
                     raise self.parse_error(UnexpectedCharError, "=")
                 else:
                     found_equals = True
-            pass
+        if not found_equals:
+            raise self.parse_error(UnexpectedCharError, self._current)
 
         if not key.sep:
             key.sep = self.extract()
@@ -411,57 +342,53 @@ class Parser:
 
         return key, val
 
-    def _parse_key(self):  # type: () -> Key
+    def _parse_key(self) -> Key:
         """
         Parses a Key at the current position;
         WS before the key must be exhausted first at the callsite.
         """
+        self.mark()
+        while self._current.is_spaces() and self.inc():
+            # Skip any leading whitespace
+            pass
         if self._current in "\"'":
             return self._parse_quoted_key()
         else:
             return self._parse_bare_key()
 
-    def _parse_quoted_key(self):  # type: () -> Key
+    def _parse_quoted_key(self) -> Key:
         """
         Parses a key enclosed in either single or double quotes.
         """
+        # Extract the leading whitespace
+        original = self.extract()
         quote_style = self._current
-        key_type = None
-        dotted = False
-        for t in KeyType:
-            if t.value == quote_style:
-                key_type = t
-                break
+        key_type = next((t for t in KeyType if t.value == quote_style), None)
 
         if key_type is None:
             raise RuntimeError("Should not have entered _parse_quoted_key()")
 
-        self.inc()
+        key_str = self._parse_string(
+            StringType.SLB if key_type == KeyType.Basic else StringType.SLL
+        )
+        if key_str._t.is_multiline():
+            raise self.parse_error(UnexpectedCharError, key_str._t.value)
+        original += key_str.as_string()
         self.mark()
-
-        while self._current != quote_style and self.inc():
+        while self._current.is_spaces() and self.inc():
             pass
-
-        key = self.extract()
-
+        original += self.extract()
+        key = SingleKey(str(key_str), t=key_type, sep="", original=original)
         if self._current == ".":
             self.inc()
-            dotted = True
-            key += "." + self._parse_key().as_string()
-            key_type = KeyType.Bare
-        else:
-            self.inc()
+            key = key.concat(self._parse_key())
 
-        return Key(key, key_type, "", dotted)
+        return key
 
-    def _parse_bare_key(self):  # type: () -> Key
+    def _parse_bare_key(self) -> Key:
         """
         Parses a bare key.
         """
-        key_type = None
-        dotted = False
-
-        self.mark()
         while (
             self._current.is_bare_key_char() or self._current.is_spaces()
         ) and self.inc():
@@ -471,85 +398,21 @@ class Parser:
         key = original.strip()
         if not key:
             # Empty key
-            raise self.parse_error(ParseError, "Empty key found")
+            raise self.parse_error(EmptyKeyError)
 
         if " " in key:
             # Bare key with spaces in it
-            raise self.parse_error(ParseError, 'Invalid key "{}"'.format(key))
+            raise self.parse_error(ParseError, f'Invalid key "{key}"')
+
+        key = SingleKey(key, KeyType.Bare, "", original)
 
         if self._current == ".":
             self.inc()
-            dotted = True
-            original += "." + self._parse_key().as_string()
-            key = original.strip()
-            key_type = KeyType.Bare
+            key = key.concat(self._parse_key())
 
-        return Key(key, key_type, "", dotted, original=original)
+        return key
 
-    def _handle_dotted_key(
-        self, container, key, value
-    ):  # type: (Union[Container, Table], Key, Any) -> None
-        names = tuple(self._split_table_name(key.as_string()))
-        name = names[0]
-        name._dotted = True
-        if name in container:
-            if not isinstance(value, Table):
-                table = Table(Container(True), Trivia(), False, is_super_table=True)
-                _table = table
-                for i, _name in enumerate(names[1:]):
-                    if i == len(names) - 2:
-                        _name.sep = key.sep
-
-                        _table.append(_name, value)
-                    else:
-                        _name._dotted = True
-                        _table.append(
-                            _name,
-                            Table(
-                                Container(True),
-                                Trivia(),
-                                False,
-                                is_super_table=i < len(names) - 2,
-                            ),
-                        )
-
-                        _table = _table[_name]
-
-                value = table
-
-            container.append(name, value)
-
-            return
-        else:
-            table = Table(Container(True), Trivia(), False, is_super_table=True)
-            if isinstance(container, Table):
-                container.raw_append(name, table)
-            else:
-                container.append(name, table)
-
-        for i, _name in enumerate(names[1:]):
-            if i == len(names) - 2:
-                _name.sep = key.sep
-
-                table.append(_name, value)
-            else:
-                _name._dotted = True
-                if _name in table.value:
-                    table = table.value[_name]
-                else:
-                    table.append(
-                        _name,
-                        Table(
-                            Container(True),
-                            Trivia(),
-                            False,
-                            is_super_table=i < len(names) - 2,
-                        ),
-                    )
-
-                    table = table[_name]
-
-    def _parse_value(self):  # type: () -> Item
+    def _parse_value(self) -> Item:
         """
         Attempts to parse a value at the current position.
         """
@@ -674,7 +537,7 @@ class Parser:
     def _parse_false(self):
         return self._parse_bool(BoolType.FALSE)
 
-    def _parse_bool(self, style):  # type: (BoolType) -> Bool
+    def _parse_bool(self, style: BoolType) -> Bool:
         with self._state:
             style = BoolType(style)
 
@@ -685,25 +548,25 @@ class Parser:
 
             return Bool(style, Trivia())
 
-    def _parse_array(self):  # type: () -> Array
+    def _parse_array(self) -> Array:
         # Consume opening bracket, EOF here is an issue (middle of array)
         self.inc(exception=UnexpectedEofError)
 
-        elems = []  # type: List[Item]
+        elems: List[Item] = []
         prev_value = None
         while True:
             # consume whitespace
             mark = self._idx
-            self.consume(TOMLChar.SPACES)
-            newline = self.consume(TOMLChar.NL)
+            self.consume(TOMLChar.SPACES + TOMLChar.NL)
             indent = self._src[mark : self._idx]
+            newline = set(TOMLChar.NL) & set(indent)
             if newline:
                 elems.append(Whitespace(indent))
                 continue
 
             # consume comment
             if self._current == "#":
-                cws, comment, trail = self._parse_comment_trail()
+                cws, comment, trail = self._parse_comment_trail(parse_trail=False)
                 elems.append(Comment(Trivia(indent, cws, comment, trail)))
                 continue
 
@@ -743,7 +606,7 @@ class Parser:
         else:
             return res
 
-    def _parse_inline_table(self):  # type: () -> InlineTable
+    def _parse_inline_table(self) -> InlineTable:
         # consume opening bracket, EOF here is an issue (middle of array)
         self.inc(exception=UnexpectedEofError)
 
@@ -779,10 +642,7 @@ class Parser:
                     raise self.parse_error(UnexpectedCharError, self._current)
 
             key, val = self._parse_key_value(False)
-            if key.is_dotted():
-                self._handle_dotted_key(elems, key, val)
-            else:
-                elems.add(key, val)
+            elems.add(key, val)
 
             # consume trailing whitespace
             mark = self._idx
@@ -799,17 +659,18 @@ class Parser:
 
         return InlineTable(elems, Trivia())
 
-    def _parse_number(self, raw, trivia):  # type: (str, Trivia) -> Optional[Item]
+    def _parse_number(self, raw: str, trivia: Trivia) -> Optional[Item]:
         # Leading zeros are not allowed
         sign = ""
         if raw.startswith(("+", "-")):
             sign = raw[0]
             raw = raw[1:]
 
-        if (
-            len(raw) > 1
-            and raw.startswith("0")
+        if len(raw) > 1 and (
+            raw.startswith("0")
             and not raw.startswith(("0.", "0o", "0x", "0b", "0e"))
+            or sign
+            and raw.startswith(".")
         ):
             return
 
@@ -829,12 +690,16 @@ class Parser:
             base = 16
 
         # Underscores should be surrounded by digits
-        clean = re.sub("(?i)(?<={})_(?={})".format(digits, digits), "", raw)
+        clean = re.sub(f"(?i)(?<={digits})_(?={digits})", "", raw).lower()
 
         if "_" in clean:
             return
 
-        if clean.endswith("."):
+        if (
+            clean.endswith(".")
+            or not clean.startswith("0x")
+            and clean.split("e", 1)[0].endswith(".")
+        ):
             return
 
         try:
@@ -845,11 +710,11 @@ class Parser:
             except ValueError:
                 return
 
-    def _parse_literal_string(self):  # type: () -> String
+    def _parse_literal_string(self) -> String:
         with self._state:
             return self._parse_string(StringType.SLL)
 
-    def _parse_basic_string(self):  # type: () -> String
+    def _parse_basic_string(self) -> String:
         with self._state:
             return self._parse_string(StringType.SLB)
 
@@ -898,12 +763,12 @@ class Parser:
 
         raise self.parse_error(InvalidCharInStringError, self._current)
 
-    def _parse_string(self, delim):  # type: (StringType) -> String
+    def _parse_string(self, delim: StringType) -> String:
         # only keep parsing for string if the current character matches the delim
         if self._current != delim.unit:
             raise self.parse_error(
                 InternalParserError,
-                "Invalid character for string type {}".format(delim),
+                f"Invalid character for string type {delim}",
             )
 
         # consume the opening/first delim, EOF here is an issue
@@ -1006,8 +871,8 @@ class Parser:
                 self.inc(exception=UnexpectedEofError)
 
     def _parse_table(
-        self, parent_name=None, parent=None
-    ):  # type: (Optional[str], Optional[Table]) -> Tuple[Key, Union[Table, AoT]]
+        self, parent_name: Optional[Key] = None, parent: Optional[Table] = None
+    ) -> Tuple[Key, Union[Table, AoT]]:
         """
         Parses a table element.
         """
@@ -1028,58 +893,28 @@ class Parser:
                 raise self.parse_error(UnexpectedEofError)
 
             is_aot = True
-
-        # Consume any whitespace
-        self.mark()
-        while self._current.is_spaces() and self.inc():
-            pass
-
-        ws_prefix = self.extract()
-
-        # Key
-        if self._current in [StringType.SLL.value, StringType.SLB.value]:
-            delimiter = (
-                StringType.SLL
-                if self._current == StringType.SLL.value
-                else StringType.SLB
-            )
-            name = self._parse_string(delimiter)
-            name = "{delimiter}{name}{delimiter}".format(
-                delimiter=delimiter.value, name=name
-            )
-
-            self.mark()
-            while self._current != "]" and self.inc():
-                if self.end():
-                    raise self.parse_error(UnexpectedEofError)
-
-                pass
-
-            ws_suffix = self.extract()
-            name += ws_suffix
-        else:
-            self.mark()
-            while self._current != "]" and self.inc():
-                if self.end():
-                    raise self.parse_error(UnexpectedEofError)
-
-                pass
-
-            name = self.extract()
-
-        name = ws_prefix + name
-
-        if not name.strip():
+        try:
+            key = self._parse_key()
+        except EmptyKeyError:
+            raise self.parse_error(EmptyTableNameError) from None
+        if self.end():
+            raise self.parse_error(UnexpectedEofError)
+        elif self._current != "]":
+            raise self.parse_error(UnexpectedCharError, self._current)
+        elif not key.key.strip():
             raise self.parse_error(EmptyTableNameError)
 
-        key = Key(name, sep="")
-        name_parts = tuple(self._split_table_name(name))
+        key.sep = ""
+        full_key = key
+        name_parts = tuple(key)
         if any(" " in part.key.strip() and part.is_bare() for part in name_parts):
-            raise self.parse_error(ParseError, 'Invalid table name "{}"'.format(name))
+            raise self.parse_error(
+                ParseError, f'Invalid table name "{full_key.as_string()}"'
+            )
 
         missing_table = False
         if parent_name:
-            parent_name_parts = tuple(self._split_table_name(parent_name))
+            parent_name_parts = tuple(parent_name)
         else:
             parent_name_parts = tuple()
 
@@ -1102,8 +937,8 @@ class Parser:
             values,
             Trivia(indent, cws, comment, trail),
             is_aot,
-            name=name,
-            display_name=name,
+            name=name_parts[0].key if name_parts else key.key,
+            display_name=full_key.as_string(),
         )
 
         if len(name_parts) > 1:
@@ -1116,36 +951,36 @@ class Parser:
                 table = Table(
                     Container(True),
                     Trivia(indent, cws, comment, trail),
-                    is_aot and name_parts[0].key in self._aot_stack,
+                    is_aot and name_parts[0] in self._aot_stack,
                     is_super_table=True,
                     name=name_parts[0].key,
                 )
 
-                result = table
-                key = name_parts[0]
+            result = table
+            key = name_parts[0]
 
-                for i, _name in enumerate(name_parts[1:]):
-                    if _name in table:
-                        child = table[_name]
-                    else:
-                        child = Table(
-                            Container(True),
-                            Trivia(indent, cws, comment, trail),
-                            is_aot and i == len(name_parts[1:]) - 1,
-                            is_super_table=i < len(name_parts[1:]) - 1,
-                            name=_name.key,
-                            display_name=name if i == len(name_parts[1:]) - 1 else None,
-                        )
+            for i, _name in enumerate(name_parts[1:]):
+                if _name in table:
+                    child = table[_name]
+                else:
+                    child = Table(
+                        Container(True),
+                        Trivia(indent, cws, comment, trail),
+                        is_aot and i == len(name_parts) - 2,
+                        is_super_table=i < len(name_parts) - 2,
+                        name=_name.key,
+                        display_name=full_key.as_string()
+                        if i == len(name_parts) - 2
+                        else None,
+                    )
 
-                    if is_aot and i == len(name_parts[1:]) - 1:
-                        table.raw_append(
-                            _name, AoT([child], name=table.name, parsed=True)
-                        )
-                    else:
-                        table.raw_append(_name, child)
+                if is_aot and i == len(name_parts) - 2:
+                    table.raw_append(_name, AoT([child], name=table.name, parsed=True))
+                else:
+                    table.raw_append(_name, child)
 
-                    table = child
-                    values = table.value
+                table = child
+                values = table.value
         else:
             if name_parts:
                 key = name_parts[0]
@@ -1155,27 +990,24 @@ class Parser:
             if item:
                 _key, item = item
                 if not self._merge_ws(item, values):
-                    if _key is not None and _key.is_dotted():
-                        self._handle_dotted_key(table, _key, item)
-                    else:
-                        table.raw_append(_key, item)
+                    table.raw_append(_key, item)
             else:
                 if self._current == "[":
-                    is_aot_next, name_next = self._peek_table()
+                    _, key_next = self._peek_table()
 
-                    if self._is_child(name, name_next):
-                        key_next, table_next = self._parse_table(name, table)
+                    if self._is_child(full_key, key_next):
+                        key_next, table_next = self._parse_table(full_key, table)
 
                         table.raw_append(key_next, table_next)
 
                         # Picking up any sibling
                         while not self.end():
-                            _, name_next = self._peek_table()
+                            _, key_next = self._peek_table()
 
-                            if not self._is_child(name, name_next):
+                            if not self._is_child(full_key, key_next):
                                 break
 
-                            key_next, table_next = self._parse_table(name, table)
+                            key_next, table_next = self._parse_table(full_key, table)
 
                             table.raw_append(key_next, table_next)
 
@@ -1189,12 +1021,12 @@ class Parser:
         if isinstance(result, Null):
             result = table
 
-            if is_aot and (not self._aot_stack or name != self._aot_stack[-1]):
-                result = self._parse_aot(result, name)
+            if is_aot and (not self._aot_stack or full_key != self._aot_stack[-1]):
+                result = self._parse_aot(result, full_key)
 
         return key, result
 
-    def _peek_table(self):  # type: () -> Tuple[bool, str]
+    def _peek_table(self) -> Tuple[bool, Key]:
         """
         Peeks ahead non-intrusively by cloning then restoring the
         initial state of the parser.
@@ -1203,7 +1035,6 @@ class Parser:
         as well as whether it is part of an AoT.
         """
         # we always want to restore after exiting this scope
-        table_name = ""
         with self._state(save_marker=True, restore=True):
             if self._current != "[":
                 raise self.parse_error(
@@ -1217,15 +1048,12 @@ class Parser:
             if self._current == "[":
                 self.inc()
                 is_aot = True
+            try:
+                return is_aot, self._parse_key()
+            except EmptyKeyError:
+                raise self.parse_error(EmptyTableNameError) from None
 
-            self.mark()
-
-            while self._current != "]" and self.inc():
-                table_name = self.extract()
-
-            return is_aot, table_name
-
-    def _parse_aot(self, first, name_first):  # type: (Table, str) -> AoT
+    def _parse_aot(self, first: Table, name_first: Key) -> AoT:
         """
         Parses all siblings of the provided table first and bundles them into
         an AoT.
@@ -1244,7 +1072,7 @@ class Parser:
 
         return AoT(payload, parsed=True)
 
-    def _peek(self, n):  # type: (int) -> str
+    def _peek(self, n: int) -> str:
         """
         Peeks ahead n characters.
 
@@ -1254,7 +1082,7 @@ class Parser:
         with self._state(restore=True):
             buf = ""
             for _ in range(n):
-                if self._current not in " \t\n\r#,]}":
+                if self._current not in " \t\n\r#,]}" + self._src.EOF:
                     buf += self._current
                     self.inc()
                     continue
@@ -1262,9 +1090,7 @@ class Parser:
                 break
             return buf
 
-    def _peek_unicode(
-        self, is_long
-    ):  # type: (bool) -> Tuple[Optional[str], Optional[str]]
+    def _peek_unicode(self, is_long: bool) -> Tuple[Optional[str], Optional[str]]:
         """
         Peeks ahead non-intrusively by cloning then restoring the
         initial state of the parser.
